module Tvdbjson
  class Episode
    include Tvdbjson::Hashable
    extend Tvdbjson::Requestable

    attr_accessor :id, :name, :series_id, :episode_number, :season_number, :first_aired, :overview

    def initialize(options = {})
      @id             = options[:id]
      @name           = options[:name]
      @series_id      = options[:series_id]
      @episode_number = options[:episode_number]
      @season_number  = options[:season_number]
      @first_aired    = options[:first_aired]
      @overview       = options[:overview]
    end

    def self.all_episodes(options = {}, authentication)
      raise Tvdbjson::RequiredSeriesIdMissingError if !options[:series_id]
      raise Tvdbjson::AuthenticationObjectMissingError unless authentication.kind_of?(Tvdbjson::Authentication)

      hash = {
        :method       => "GET",
        :uri          => "/series/#{options[:series_id]}/episodes",
        :params       => { "page" => 1 }
      }

      # Create an empty array for all episodes
      arr = []

      # Receive the first page of results
      response = send_authenticated_request(hash, authentication)

      # Store the Page count to retrieve additional pages in the future
      page_count = get_page_count_from_response(response)

      # Whilst we have the response object for page 1, let's add it to our array
      arr.concat( Episode.from_response(response,options[:series_id]) )

      # Now to process the additional pages (starting with page 2)
      (2..page_count).each do |page_number|
        hash = {
          :method       => "GET",
          :uri          => "/series/#{options[:series_id]}/episodes",
          :params       => { "page" => page_number }
        }
        response = send_authenticated_request(hash, authentication)
        arr.concat( Episode.from_response(response,options[:series_id]) )
      end

      arr

    end

    def self.search_by_season_and_episode(options = {}, authentication)
      raise Tvdbjson::RequiredSeriesIdMissingError if !options[:series_id]
      raise Tvdbjson::RequiredSearchParamMissingError if !options[:season_number]
      raise Tvdbjson::RequiredSearchParamMissingError if !options[:episode_number]
      raise Tvdbjson::AuthenticationObjectMissingError unless authentication.kind_of?(Tvdbjson::Authentication)

      hash = {
        :method       => "GET",
        :uri          => "/series/#{options[:series_id]}/episodes/query",
        :after_action => "Tvdbjson::Episode.from_response(response, #{options[:series_id]})",
        :params       => { "airedSeason" => options[:season_number], "airedEpisode" => options[:episode_number] }
      }
      send_authenticated_request(hash, authentication)
    end

    def self.from_response(response, series_id)
      results_array = []

      begin
        response.parsed_response["data"].each do |record|
          hash                  = {}
          hash[:id]             = record["id"]
          hash[:name]           = record["episodeName"]
          hash[:series_id]      = series_id
          hash[:episode_number] = record["airedEpisodeNumber"]
          hash[:season_number]  = record["airedSeasonID"]
          hash[:first_aired]    = record["firstAired"]
          hash[:overview]       = record["overview"]

          result = Episode.new(hash)
          results_array << result
        end
      rescue NoMethodError
        # Means an empty result set was found, don't do anything special
        # here as this method will return an empty array.
      end
      results_array
    end

    def self.get_page_count_from_response(response)
      response.parsed_response["links"]["last"]
    end

  end
end
